/**
 * \file IfxGeth_Eth_Phy_Rtl8211f.c
 * \brief Source file of ETHERNET PHY RTL8211F communication
 *
 * \copyright Copyright (c) 2019 Infineon Technologies AG. All rights reserved.
 *
 *
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Use of this file is subject to the terms of use agreed between (i) you or
 * the company in which ordinary course of business you are acting and (ii)
 * Infineon Technologies AG or its licensees. If and as long as no such
 * terms of use are agreed, use of this file is subject to following:


 * Boost Software License - Version 1.0 - August 17th, 2003

 * Permission is hereby granted, free of charge, to any person or
 * organization obtaining a copy of the software and accompanying
 * documentation covered by this license (the "Software") to use, reproduce,
 * display, distribute, execute, and transmit the Software, and to prepare
 * derivative works of the Software, and to permit third-parties to whom the
 * Software is furnished to do so, all subject to the following:

 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer, must
 * be included in all copies of the Software, in whole or in part, and all
 * derivative works of the Software, unless such copies or derivative works are
 * solely in the form of machine-executable object code generated by a source
 * language processor.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.

 *
 */

/******************************************************************************/
/*----------------------------------Includes----------------------------------*/
/******************************************************************************/

#include "IfxGeth_Phy_Rtl8211f.h"
#include "Ifx_Console.h"



#define IFXGETH_PHY_RTL8211F_WAIT_MDIO_READY() while (GETH_MAC_MDIO_ADDRESS.B.GB) {}

/******************************************************************************/
/*--------------------------------Enumerations--------------------------------*/
/******************************************************************************/

/******************************************************************************/
/*-----------------------------Data Structures--------------------------------*/
/******************************************************************************/

/******************************************************************************/
/*------------------------------Global variables------------------------------*/
/******************************************************************************/
#if CPU_WHICH_SERVICE_ETHERNET == 0
    #if defined(__GNUC__)
    #pragma section ".text_cpu0" ax
    #pragma section ".bss_cpu0" awc0
    #endif
    #if defined(__TASKING__)
    #pragma section code    "text_cpu0"
    #pragma section farbss  "bss_cpu0"
    #pragma section fardata "data_cpu0"
    #endif
    #if defined(__DCC__)
    #pragma section CODE ".text_cpu0"
    #pragma section DATA ".data_cpu0" ".bss_cpu0" far-absolute RW
    #endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu0"
    #pragma ghs section bss= ".bss_cpu0"
    #pragma ghs section data=".data_cpu0"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 1) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
    #if defined(__GNUC__)
    #pragma section ".text_cpu1" ax
    #pragma section ".bss_cpu1" awc1
    #endif
    #if defined(__TASKING__)
    #pragma section code    "text_cpu1"
    #pragma section farbss  "bss_cpu1"
    #pragma section fardata "data_cpu1"
    #endif
    #if defined(__DCC__)
    #pragma section CODE ".text_cpu1"
    #pragma section DATA ".data_cpu1" ".bss_cpu1" far-absolute RW
    #endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu1"
    #pragma ghs section bss= ".bss_cpu1"
    #pragma ghs section data=".data_cpu1"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 2) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
    #if defined(__GNUC__)
    #pragma section ".text_cpu2" ax
    #pragma section ".bss_cpu2" awc2
    #endif
    #if defined(__TASKING__)
    #pragma section code    "text_cpu2"
    #pragma section farbss  "bss_cpu2"
    #pragma section fardata "data_cpu2"
    #endif
    #if defined(__DCC__)
    #pragma section CODE ".text_cpu2"
    #pragma section DATA ".data_cpu2" ".bss_cpu2" far-absolute RW
    #endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu2"
    #pragma ghs section bss= ".bss_cpu2"
    #pragma ghs section data=".data_cpu2"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 3) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
	#if defined(__GNUC__)
    #pragma section ".text_cpu3" ax
	#pragma section ".bss_cpu3" awc3
	#endif
	#if defined(__TASKING__)
    #pragma section code    "text_cpu3"
    #pragma section farbss  "bss_cpu3"
    #pragma section fardata "data_cpu3"
	#endif
	#if defined(__DCC__)
    #pragma section CODE ".text_cpu3"
	#pragma section DATA ".data_cpu3" ".bss_cpu3" far-absolute RW
	#endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu3"
    #pragma ghs section bss= ".bss_cpu3"
    #pragma ghs section data=".data_cpu3"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 4) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
	#if defined(__GNUC__)
    #pragma section ".text_cpu4" ax
	#pragma section ".bss_cpu4" awc4
	#endif
	#if defined(__TASKING__)
    #pragma section code    "text_cpu4"
    #pragma section farbss  "bss_cpu4"
    #pragma section fardata "data_cpu4"
	#endif
	#if defined(__DCC__)
    #pragma section CODE ".text_cpu4"
	#pragma section DATA ".data_cpu4" ".bss_cpu4" far-absolute RW
	#endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu4"
    #pragma ghs section bss= ".bss_cpu4"
    #pragma ghs section data=".data_cpu4"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 5) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
	#if defined(__GNUC__)
    #pragma section ".text_cpu5" ax
	#pragma section ".bss_cpu5" awc5
	#endif
	#if defined(__TASKING__)
    #pragma section code    "text_cpu5"
    #pragma section farbss  "bss_cpu5"
    #pragma section fardata "data_cpu5"
	#endif
	#if defined(__DCC__)
    #pragma section CODE ".text_cpu5"
	#pragma section DATA ".data_cpu5" ".bss_cpu5" far-absolute RW
	#endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu5"
    #pragma ghs section bss= ".bss_cpu5"
    #pragma ghs section data=".data_cpu5"
    #endif
#else
#error "Set CPU_WHICH_SERVICE_ETHERNET to a valid value!"
#endif

/******************************************************************************/
/*-----------------------Exported Variables/Constants-------------------------*/
/******************************************************************************/

/****************** Port ********************/
#define ETH_88Q6113_J5_1_PORT_0xE   0xE0007  // 917,511 origin_07
#define ETH_88Q6113_J5_1_PORT_0xF   0xF0007  // 917,511 origin_07
#define ETH_88Q6113_MCU_PORT_0xE    0xE0008  // 917,512 origin_08
#define ETH_88Q6113_MCU_PORT_0xF    0xF0008  // 917,512 origin_08
#define ETH_88Q6113_GLOBAL1_PORT_E  0xE001B  // 917,531 origin_27
#define ETH_88Q6113_GLOBAL1_PORT_F  0xF001B  // 917,531 origin_27
#define ETH_88Q6113_X9_PORT         0xE000B  // 917,515 origin_11

/****************** Register ********************/
#define ETH_88Q6113_PORT_STATUS_REG               0x00
#define ETH_88Q6113_PORT_PHYSICAL_CONTROL_REG     0x01
#define ETH_88Q6113_ID_REG                        0x03


/******************************************************************************/
/*-------------------------Function Prototypes--------------------------------*/
/******************************************************************************/
void IfxGeth_88Q6113_Clause22_readMDIORegister(uint32 Port_Switch_Addr, uint32 regAddr, uint32 *pData)
{
     uint32  Switch_Addr = 0xFF0000 & Port_Switch_Addr;
     uint32  Port_Addr = 0x000000FF & Port_Switch_Addr;
     uint32  Reg_Operand = (0x1800 | Port_Addr << 5 | regAddr);
     uint32  Execute_Instruction = (0x8000 | Reg_Operand) ;

     IfxGeth_Eth_Phy_Rtl8211f_write_mdio_reg((Switch_Addr >> 16), 0, Reg_Operand);//д����
     IfxGeth_Eth_Phy_Rtl8211f_write_mdio_reg((Switch_Addr >> 16), 0, Execute_Instruction);//��Ч
     IfxGeth_Eth_Phy_Rtl8211f_read_mdio_reg((Switch_Addr >> 16), 1, pData);//�����ݼĴ���
}

void IfxGeth_88Q6113_Clause22_writeMDIORegister(uint32 Port_Switch_Addr, uint32 regAddr, uint32 data)
{
     uint32  Switch_Addr = 0xFF0000 & Port_Switch_Addr;
     uint32  Port_Addr = 0x000000FF & Port_Switch_Addr;
     uint32  Reg_Operand = (0x1400 | Port_Addr << 5 | regAddr);
     uint32  Execute_Instruction = (0x8000 | Reg_Operand);

     IfxGeth_Eth_Phy_Rtl8211f_write_mdio_reg((Switch_Addr >> 16), 1, data);//д���ݼĴ���
     IfxGeth_Eth_Phy_Rtl8211f_write_mdio_reg((Switch_Addr >> 16), 0, Reg_Operand);//д����
     IfxGeth_Eth_Phy_Rtl8211f_write_mdio_reg((Switch_Addr >> 16), 0, Execute_Instruction);//��Ч
}
/******************************************************************************/
/*-------------------------Function Implementations---------------------------*/
/******************************************************************************/

uint32 IfxGeth_Eth_Phy_Rtl8211f_init(void)
{
    uint32 value=0;
    IFXGETH_PHY_RTL8211F_WAIT_MDIO_READY();


    /* ����PHYоƬͨѶ�Ƿ����� *** Add by Turing, 2022.3.1 */
    value=0;
    IfxGeth_88Q6113_Clause22_readMDIORegister(0xe0008, 0x03, &value);
    Ifx_Console_print("6113_A_Port_8_ID:%X\r\n",value);
     value=0;
     IfxGeth_88Q6113_Clause22_readMDIORegister(0xe0008, 0x03, &value);
     Ifx_Console_print("6113_B_Port_8_ID:%X\r\n\n",value);
     value=0;



     /* Global reg config*/
       value = 0x1000;
       //0001 0111 0111 1011
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_E, 27, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_F, 27, value);

       value = 0x8100;
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_E, 26, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_F, 26, value);

       value = 0xfce0;
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_E, 25, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_F, 25, value);
       value = 0;


       /* set MCU_Port7 RGMII 1000M FullDuplex */
       value = 0xde07;
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_MCU_PORT_0xE, ETH_88Q6113_PORT_STATUS_REG, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_MCU_PORT_0xF, ETH_88Q6113_PORT_STATUS_REG, value);
       value = 0;



       /* set MCU_Port8 RGMII 1000M FullDuplex */
       value = 0x0e07;
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_J5_1_PORT_0xE, ETH_88Q6113_PORT_STATUS_REG, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_J5_1_PORT_0xF, ETH_88Q6113_PORT_STATUS_REG, value);
       value = 0;

       /* Global reg config*/
       value = 0x1000;
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_E, 27, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_F, 27, value);
       value = 0x9000;
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_E, 26, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_F, 26, value);
       value = 0xfce8;
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_E, 25, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_GLOBAL1_PORT_F, 25, value);

       /* open TXdelay Rxdelay */
       value = 0xc003;
//       /* MCU_6113_RGMII Port8 */
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_MCU_PORT_0xE, ETH_88Q6113_PORT_PHYSICAL_CONTROL_REG, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_MCU_PORT_0xF, ETH_88Q6113_PORT_PHYSICAL_CONTROL_REG, value);

       /*J5_1_6113_RGMII Port7 */
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_J5_1_PORT_0xE, ETH_88Q6113_PORT_PHYSICAL_CONTROL_REG, value);
       IfxGeth_88Q6113_Clause22_writeMDIORegister(ETH_88Q6113_J5_1_PORT_0xF, ETH_88Q6113_PORT_PHYSICAL_CONTROL_REG, value);
       value = 0;


    return 1;
}


void IfxGeth_Eth_Phy_Rtl8211f_read_mdio_reg(uint32 layeraddr, uint32 regaddr, uint32 *pdata)
{
    // 5bit Physical Layer Adddress, 5bit GMII Regnr, 4bit csrclock divider, Read, Busy
	GETH_MAC_MDIO_ADDRESS.U = (layeraddr << 21) | (regaddr << 16) | (0 << 8) | (3 << 2) | (1 << 0);

	IFXGETH_PHY_RTL8211F_WAIT_MDIO_READY();

    // get data
    *pdata = GETH_MAC_MDIO_DATA.U;
}


void IfxGeth_Eth_Phy_Rtl8211f_write_mdio_reg(uint32 layeraddr, uint32 regaddr, uint32 data)
{
    // put data
	GETH_MAC_MDIO_DATA.U = data;

    // 5bit Physical Layer Adddress, 5bit GMII Regnr, 4bit csrclock divider, Write, Busy
    GETH_MAC_MDIO_ADDRESS.U = (layeraddr << 21) | (regaddr << 16) | (0 << 8) |  (1 << 2) | (1 << 0);

    IFXGETH_PHY_RTL8211F_WAIT_MDIO_READY();
}

#if defined(__GNUC__)
#pragma section // end text section
#endif
#if defined(__TASKING__)
#pragma section code restore
#pragma section fardata restore
#pragma section farbss restore
#endif
#if defined(__DCC__)
#pragma section CODE
#pragma section DATA RW
#endif
#if defined(__ghs__)
#pragma ghs section text=default
#pragma ghs section data=default
#pragma ghs section bss=default
#endif
