
#include "TCP_Server.h"

struct TCPServer_Com_typ TCPServer_Com;
unsigned long TCP_TickCount_1ms_Pre;

/*********************************************************************************************************************/
/*-------------------------------------------------Global variables--------------------------------------------------*/
/*********************************************************************************************************************/
/* Infineon Logo to be printed on the terminal at every new connection */
char * firstLogo =
"                   *20221108******\r\n\0";

//tcpPcb *g_echoPcb;                                                  /* Pointer to the TCP protocol control block    */

/*********************************************************************************************************************/
/*------------------------------------------------Function Prototypes------------------------------------------------*/
/*********************************************************************************************************************/
err_t TcpAccept (void *arg, tcpPcb *newPcb, err_t err);            /* Accept callback is called every time a client establishes a new connection                   */
err_t TcpRecv (void *arg, tcpPcb *tpcb, pBuf *p, err_t err);       /* Recv callback is called every time data is received through the TCP connection               */
void TcpError (void *arg, err_t err);                              /* Error callback is called if a fatal error has already occurred on the connection             */
err_t TcpPoll (void *arg, tcpPcb *tpcb);                           /* Poll function is called periodically by the TCP stack                                        */
err_t TcpSent (void *arg, tcpPcb *tpcb, u16_t len);                /* Sent callback is called when TCP data has successfully been delivered to the remote host     */
void TcpSend (tcpPcb *tPcb, EchoSession *es);                      /* Send function enqueues TCP data to be delivered to the remote client                         */
void TcpUnpack (tcpPcb *tPcb, EchoSession *es);                    /* Unpack function dequeues data from the package buffer and copies it in the session storage   */
void TcpClose (tcpPcb *tPcb, EchoSession *es);                     /* Close function closes a TCP connection and deallocates session resources                     */

void TcpServerInit(tcpPcb *newPcb, TCPServer_Com_typ *com);
err_t TcpServerSend(tcpPcb *newPcb, unsigned char *data, uint16_t length);
err_t tcp_server_senddata(tcpPcb *tpcb, pBuf *p);
/*********************************************************************************************************************/
/*---------------------------------------------Function Implementations----------------------------------------------*/
/*********************************************************************************************************************/
void TCP_Init(void)
{
    TCP_TickCount_1ms_Pre = g_TickCount_1ms;
    memset(&TCPServer_Com,0,sizeof(TCPServer_Com));

    //服务器端口设置
    TCPServer_Com.port = TCP_CONNECT_PORT;
    TCPServer_Com.step = 1;

}

void TCPServerConnect(void)
{
    switch(TCPServer_Com.step)
    {
        case 1:
            TcpServerInit(TCPServer_Com.tcpPcb,&TCPServer_Com);
            break;
        case 2:  //等待client连接,回调函数TcpAccept内处理
            break;
        case 3:
        {
            //等待数据接收，回调函数TcpRecv内处理,接收到数据后进行数据发送
            if(TCPServer_Com.TCP_RecvDataCount != TCPServer_Com.TCP_RecvDataCount_Pre)
            {
                TCPServer_Com.TCP_RecvDataCount_Pre = TCPServer_Com.TCP_RecvDataCount;

//                err_t retErr = TcpServerSend(TCPServer_Com.tcpPcbConnect, TCPServer_Com.TCP_SendBuffer, TCPServer_Com.TCP_SendDataLength);
//                if(retErr != ERR_OK)
//                {
//                    Ifx_Console_print("TCP send data length is %i,Err %i,\r\n",TCPServer_Com.TCP_SendDataLength,retErr);
//                }
                TCPServer_Com.RingBuffer_UntreatedLength = 0;
                TCPServer_Com.RingBuffer_HeadIndex = 0;
            }
            if(g_TickCount_1ms % 50 == 0)
            {
                TCPServer_Com.TCP_SendDataLength = 2000;
                TCPServer_Com.TCP_SendBuffer[1996] = 0xAA;
                TCPServer_Com.TCP_SendBuffer[1997] = 0xAA;
                TCPServer_Com.TCP_SendBuffer[1998] = 0xBB;
                TCPServer_Com.TCP_SendBuffer[1999] = 0xBB;
                err_t retErr = TcpServerSend(TCPServer_Com.tcpPcbConnect, TCPServer_Com.TCP_SendBuffer, TCPServer_Com.TCP_SendDataLength);
               if(retErr != ERR_OK)
               {
                   Ifx_Console_print("TCP send data length is %i,Err %i,\r\n",TCPServer_Com.TCP_SendDataLength,retErr);
               }
            }
        }
            break;
        case 4:
            break;
        default:
            break;
    }

//    Ifx_Console_print("TCP step %i\r\n",TCPServer_Com.step);  //20221108


}

/* Function to initialize the ECHO program */
void TcpServerInit(tcpPcb *newPcb, TCPServer_Com_typ *com)
{
    newPcb = tcp_new();                                          /* Create a new TCP protocol control block                                                      */
    if (newPcb != NULL)                                          /* If the creation was successful...                                                            */
    {
        err_t err = tcp_bind(newPcb, IP_ADDR_ANY, com->port);           /* ...bind the TCP procotol control block to any local address and local port 80.               */
        if (err == ERR_OK)                                          /* If the binding was successful...                                                             */
        {
            Ifx_Console_print("TCP bind successful\r\n");
            newPcb = tcp_listen(newPcb);                      /* ...set the TCP control block able to accept incoming connections.                            */
            tcp_accept(newPcb, TcpAccept);                      /* Configure the callback function to be called when a new connection is established.           */
            com->step = 2;
        }
        else
        {
            Ifx_Console_print("TCP: unable to bind to any address on port %i.\r\n",com->port);
        }
    }
    else
    {
        Ifx_Console_print("TCP: unable to create a TCP control block.\r\n");
    }
}

err_t TcpServerSend(tcpPcb *newPcb, unsigned char *data, uint16_t length)
{
//    Ifx_Console_print("TCP Send data length:%i\r\n",length);
    err_t retErr;

    pBuf *ptr;
//    pBuf *ptr_mid;
    ptr = pbuf_alloc(PBUF_TRANSPORT, length,PBUF_POOL);  //申请内存
    pbuf_take(ptr,(char*)data,length);   //将tcp_server_sentbuf[]中的数据拷贝到es->p_tx中
    retErr = tcp_server_senddata(newPcb,ptr);   //将tcp_server_sentbuf[]里面复制给pbuf的数据发送出去
//    while(ptr != NULL)
//    {
//        ptr_mid = ptr;
//        Ifx_Console_print("TCP Send data:%i,%i\r\n",length,ptr_mid->len);
//        ptr = ptr_mid->next;
//        pbuf_free(ptr_mid);
//    }

//    if(ptr != NULL)
//    {
//        pbuf_free(ptr);//释放内存
//        Ifx_Console_print("I am in pbuf_free(ptr)\r\n");
//    }

//
//    unsigned short sendDataLength = 0;
//    retErr = ERR_OK;
//    while(length-sendDataLength > 350 && retErr == ERR_OK)
//    {
//
//        retErr = tcp_write(newPcb, data+sendDataLength,350,1);
//        sendDataLength += 350;
//        Ifx_Console_print("TCP Send 'while' len:%i,Err:%i \r\n",sendDataLength,retErr);
//    }
//    retErr = tcp_write(newPcb, data+sendDataLength,length-sendDataLength,1);

    return retErr;

}

//此函数用来发送数据
err_t tcp_server_senddata(tcpPcb *tpcb, pBuf *p)
{
    struct pbuf *ptr;
    unsigned short plen;
    err_t wr_err = ERR_OK;
    while((wr_err == ERR_OK) &&
            p != NULL &&
            (p->len <= tcp_sndbuf(tpcb)))
    {
//        Ifx_Console_print("TCP Send start 'while' len:%i %i \r\n",p->len,tcp_sndbuf(tpcb));
        ptr = p;
        wr_err = tcp_write(tpcb,ptr->payload,ptr->len,1);
        if(wr_err == ERR_OK)
        {
            plen = ptr->len;
            p = ptr->next;            //指向下一个pbuf

            if (p != NULL)
            {
                pbuf_ref(p);
            }
            u8_t freed;
            do
            {
                freed = pbuf_free(ptr);
//                Ifx_Console_print("TCP Send pbuf_free(ptr):%i \r\n",freed);
            } while (freed == 0);
            tcp_recved(tpcb, plen);
        }
        else if(wr_err == ERR_MEM)
        {
            p = ptr;
        }
        tcp_output(tpcb);               //将发送缓冲队列中的数据立即发送出去
//        if (p != NULL)
//        {
//            Ifx_Console_print("TCP Send end 'while' len:%i %i,Err:%i \r\n",p->len,tcp_sndbuf(tpcb),wr_err);
//        }
    }

    return wr_err;

}


/* Accept callback: it is called every time a client establish a new connection */
err_t TcpAccept(void *arg, tcpPcb *newPcb, err_t err)
{
    Ifx_Console_print("TCP is 'TcpAccept()' \r\n");
    LWIP_UNUSED_ARG(arg);                                               /* Eliminate compiler warning about unused arguments                                        */
    LWIP_UNUSED_ARG(err);                                               /* Eliminate compiler warning about unused arguments                                        */

    err_t retErr;                                                       /* Allocate memory for function return value                                                */
    EchoSession *es = (EchoSession*) mem_malloc(sizeof(EchoSession));   /* Allocate memory for the session data                                                     */
    if (es != NULL)                                                     /* If memory allocation was successful the session can be initialized                       */
    {
        es->state = ES_ACCEPTED;                                        /* The new session has been accepted                                                        */
        es->pcb = newPcb;                                               /* A pointer to the TCP control block to use for this session is saved in the session       */
        es->p = NULL;                                                   /* The packet buffer for received data is initially empty                                   */
        memset(es->storage, 0, STORAGE_SIZE_BYTES);                     /* Allocate memory for storing processed received data                                      */
        es->nextFreeStoragePos = 0;                                     /* The first free position in the received data array is the beginning of the array         */

        tcp_arg(newPcb, es);                                            /* Specify that the session shall be passed as argument to every callback function          */
        tcp_recv(newPcb, TcpRecv);                                     /* Configure the callback function to be called when new data is received                   */
        tcp_sent(newPcb, TcpSent);                                     /* Configure the callback function to be called when new data is sent                       */
        tcp_err(newPcb, TcpError);                                     /* Configure the callback function to be called when a fatal connection error occurs        */
        tcp_poll(newPcb, TcpPoll, 0);                                  /* Configure the callback function to be periodically called by TCP
                                                                         * The time interval is specified as multiple of the TCP coarse timer interval, which is
                                                                     * called twice a second                                                                    */
        retErr = ERR_OK;                                                /* Set the return value when no error occured                                               */

        TCPServer_Com.tcpPcbConnect = newPcb;
        TCPServer_Com.step = 3;
        tcp_write(newPcb, firstLogo, strlen(firstLogo), 1);                   /* Send the Infineon logo to the remote client                                              */
    }
    else                                                                /* If it was not possible to allocate the necessary memory for the session...               */
    {
        retErr = ERR_MEM;                                               /* ... set the return value to "Out of memory".                                             */
    }
    return retErr;                                                      /* Return result                                                                            */
}

/* Recv callback: it is called every time data is received through the TCP connection */
err_t TcpRecv (void *arg, tcpPcb *tpcb, pBuf *p, err_t err)
{
//    Ifx_Console_print("TCP is 'TcpRecv' \r\n");
    err_t retErr;                                                       /* Allocate memory for function return value                                                */
    EchoSession *es = (EchoSession*) arg;                               /* Get a pointer to the current session                                                     */
//    pBuf *q;
    if (p == NULL)                                                      /* If there is no enqueued received data after the RECV callback was called, it means the   */
    {                                                                   /* remote client closed the connection in the meanwhile                                     */
        es->state = ES_CLOSING;                                         /* Set the state of this session to CLOSING in order to free its resources                  */
        if (es->p == NULL)                                              /* If the session does not have any leftover unprocessed data...                            */
        {
            TcpClose(tpcb, es);                                        /* ... close the session and free its resources.                                            */
        }
        else                                                            /* If the session does have leftover unprocessed data...                                    */
        {
            TcpUnpack(tpcb, es);                                       /* ... process unprocessed data ...                                                         */
            TcpSend(tpcb, es);                                         /* ... and send processed data.                                                             */
            Ifx_Console_print("TCP is 'p == NULL' \r\n");
        }
        retErr = ERR_OK;                                                /* Signal a successful outcome                                                              */
    }
    else if (err != ERR_OK)                                             /* If there was an error while receiving data...                                            */
    {
        /* Cleanup, for unkown reason */
        if (p != NULL)                                                  /* ... if some data was received and stored...                                              */
        {
            es->p = NULL;                                               /* ... invalidate the unprocessed data contained in the session.                            */
            pbuf_free(p);                                               /* Dereference and deallocate last received data which is probably corrupted                */
        }
        retErr = err;                                                   /* Propagate the error                                                                      */
    }
    else if (es->state == ES_ACCEPTED)                                  /* If the current session is in state ACCEPTED...                                           */
    {
        es->state = ES_RECEIVING;                                       /* ... change its state to RECEIVING.                                                       */
        es->p = p;                                                      /* Set the unprocessed data buffer of the session to the received one                       */
        TcpUnpack(tpcb, es);                                           /* Perform a first incoming data processing                                                 */
//        TcpSend(tpcb, es);                                             /* Send the first echo to remote client                                                     */
        es->nextFreeStoragePos = 0;
        Ifx_Console_print("TCP is 'es->state == ES_ACCEPTED' \r\n");
        retErr = ERR_OK;                                                /* Signal a successful outcome                                                              */
    }
    else if (es->state == ES_RECEIVING)                                 /* If the current session was already receiving data...                                     */
    {
        if (es->p == NULL)                                              /* ... and no unprocessed data is contained in the session...                               */
        {
            es->p = p;                                                  /* ... set the unprocessed data buffer of the session to the received one                   */
//            unsigned char flags = es->p->flags;

            TcpUnpack(tpcb, es);                                       /* Process the incoming data                                                                */
//            TcpSend(tpcb, es);                                         /* Send an echo to the remote client                                                        */
            //TCP数据拆包标识位
//            if(flags == 1)
            {
                TCPServer_Com.TCP_RecvDataCount ++;
                unsigned char msg_Buffer[TCP_RECV_BUF_LENGTH_MAX] = {0};
                uint16_t recvLength = 0;
                //循环读取TCP数据
                recvLength = (es->nextFreeStoragePos < TCP_RECV_BUF_LENGTH_MAX) ? es->nextFreeStoragePos : TCP_RECV_BUF_LENGTH_MAX;
                memcpy(msg_Buffer,es->storage,recvLength);
                for(uint16_t i = 0; i < recvLength; i ++)
                {
                    TCPServer_Com.RingBuffer[TCPServer_Com.RingBuffer_HeadIndex] = msg_Buffer[i];
                    TCPServer_Com.RingBuffer_HeadIndex ++;
                    TCPServer_Com.RingBuffer_HeadIndex = TCPServer_Com.RingBuffer_HeadIndex % RING_BUFFER_LENGTH_MAX;
                    TCPServer_Com.RingBuffer_UntreatedLength ++;
                }

                es->nextFreeStoragePos = 0;

                Ifx_Console_print("TCP receive data length is %i,g_TickCount(%i),delta(%i ms),\r\n",TCPServer_Com.RingBuffer_UntreatedLength,g_TickCount_1ms, g_TickCount_1ms-TCP_TickCount_1ms_Pre);
                TCP_TickCount_1ms_Pre = g_TickCount_1ms;
            }
        }
        else
        {
            pBuf *ptr = es->p;                                          /* ... create a local reference for the received data                                       */
            pbuf_chain(ptr, p);                                         /* Chain the old unprocessed data contained in the session with the new one                 */
        }

        retErr = ERR_OK;                                             /* Signal a successful outcome                                                           */

    }
    else                                                                /* If we got new incoming data in an unknown state, discard the data                        */
    {
        tcp_recved(tpcb, p->tot_len);                                   /* Mark the incoming data as received                                                       */
        es->p = NULL;                                                   /* Invalidate the unprocessed data contained in the session                                 */
        pbuf_free(p);                                                   /* Dereference and deallocate last received data                                            */
        retErr = ERR_OK;                                                /* Signal a successful outcome                                                              */
    }
    return retErr;                                                      /* Return result                                                                            */
}

/* Error callback: it is called if a fatal error has already occurred on the connection */
void TcpError(void *arg, err_t err)
{
    Ifx_Console_print("TCP is 'TcpError()' \r\n");
    LWIP_UNUSED_ARG(err);                                               /* Eliminate compiler warning about unused arguments                                        */

    EchoSession *es = (EchoSession*) arg;                               /* Get a pointer to the current session                                                     */

    if (es != NULL)                                                     /* If a session exists we cannot do anything with it anymore, since the fatal error         */
    {                                                                   /* occurred ...                                                                             */
        mem_free(es);                                                   /* ... free memory assigned to the session                                                  */
    }
}

/* Poll function: it is called periodically by the TCP stack */
err_t TcpPoll(void *arg, tcpPcb *tpcb)
{
    err_t retErr;                                                       /* Allocate memory for function return value                                                */
    EchoSession *es = (EchoSession*) arg;                               /* Get a pointer to the current session                                                     */

    if (es != NULL)                                                     /* If a session exists ...                                                                  */
    {
//        Ifx_Console_print("TCP is 'TcpPoll' \r\n");
        if (es->p != NULL ||                                            /* ... and if the session has any leftover unprocessed data...                              */
            es->nextFreeStoragePos != 0)                                /* ... or there is still data in the session storage to send back...                        */
        {
            TcpUnpack(tpcb, es);                                       /* ... process unprocessed data ...                                                         */
//            TcpSend(tpcb, es);                                         /* ... and send processed data.                                                             */
            Ifx_Console_print("TCP is 'TcpPoll' :%i\r\n",es->nextFreeStoragePos);
        }
        else                                                            /* If the session has no leftover unprocessed data...                                       */
        {
            if (es->state == ES_CLOSING)                                /* ... and the session was marked for being closed...                                       */
            {
                TcpClose(tpcb, es);                                    /* ... close the session and free its resources.                                            */
            }
        }
        retErr = ERR_OK;                                                /* Signal a successful outcome                                                              */
    }
    else                                                                /* If no session exists something went wrong, the TCP connection has to be aborted          */
    {
        tcp_abort(tpcb);                                                /* Abort the TCP connection                                                                 */
        retErr = ERR_ABRT;                                              /* Propagate the abort error                                                                */
    }
    return retErr;                                                      /* Return result                                                                            */
}

/* Sent callback: it is called when TCP data has successfully been delivered to the remote host */
err_t TcpSent(void *arg, tcpPcb *tpcb, u16_t len)
{
    LWIP_UNUSED_ARG(len);                                               /* Eliminates compiler warning about unused arguments                                       */

    EchoSession *es = (EchoSession*) arg;                               /* Get a pointer to the current session                                                     */

//    Ifx_Console_print("TCP is 'TcpSent':%i \r\n",es->nextFreeStoragePos);
    if (es->p != NULL )                                                 /* If the session has any leftover unprocessed data...                                      */
    {
        TcpUnpack(tpcb, es);                                           /* ... process unprocessed data ...                                                         */
        TcpSend(tpcb, es);                                             /* ... and send processed data.                                                             */
    }
    else                                                                /* If the session has no leftover unprocessed data...                                       */
    {
        if (es->state == ES_CLOSING)                                    /* ... and the session was marked for being closed...                                       */
        {
            TcpClose(tpcb, es);                                        /* ... close the session and free its resources.                                            */
        }
    }
    return ERR_OK;                                                      /* Return successful result                                                                 */
}

/* Send function: enqueues TCP data to be delivered to the remote client */
void TcpSend(tcpPcb *tpcb, EchoSession *es)
{
    if(es->nextFreeStoragePos == 0)                                     /* If the next free position in the storage array is the beginning of the array, there is   */
    {                                                                   /* nothing to send                                                                          */
        return;
    }
//    if(es->storage[es->nextFreeStoragePos - 1] != '\n' &&               /* If the string to be sent does not end with a new line...                                 */
//       es->nextFreeStoragePos < STORAGE_SIZE_BYTES)                     /* ... and the storage is not full ...                                                      */
//    {                                                                   /* ... do not send yet. Wait for the string to be echoed to terminate                       */
//        return;                                                         /* or the storage to completely fill.                                                       */
//    }
//    err_t wrErr = tcp_write(tpcb, "Board: ", 7, 1);                     /* Enqueue an echo preamble to be sent to the remote client                                 */
    err_t wrErr = tcp_write(tpcb, es->storage, es->nextFreeStoragePos, 1);   /* Enqueue the string stored in the session for sending                                     */
    if(wrErr == ERR_OK)                                                 /* If data was correctly enqueued for TCP transmission ...                                  */
    {
        es->nextFreeStoragePos = 0;                                     /* ... remove sent data from the session storage.                                           */
    }
}

/* Unpack function: dequeues data from the package buffer and copies it in the session storage */
void TcpUnpack(tcpPcb *tpcb, EchoSession *es)
{
    pBuf *ptr;                                                          /* Local reference to a packet buffer                                                       */

    while ((es->p != NULL))                                             /* Process data while there still are packets to process                                    */
    {
        ptr = es->p;                                                    /* Local reference of the packet to process                                                 */

        if (es->nextFreeStoragePos + ptr->len <= STORAGE_SIZE_BYTES)    /* If there is enough space in the session storage for the new data ...                     */
        {
            memcpy(&es->storage[es->nextFreeStoragePos],                /* ... copy data in the session storage                                                     */
                    ptr->payload,
                    ptr->len);
            es->nextFreeStoragePos += ptr->len;                         /* Increase the index of the new free position in the session storage array                 */

//            u16_t plen = ptr->len;                                      /* Number of bytes successfully received and stored                                         */
            u16_t plen = ptr->tot_len;

            es->p = ptr->next;                                          /* Get the next packet in the chain (if any)                                                */
            if (es->p != NULL)                                          /* If there is another packet to be processed...                                            */
            {
                pbuf_ref(es->p);                                        /* Inform the LwIP framework that the packet to be processed is linked to the session,      */
            }                                                           /* and shall not be disposed                                                                */

            u8_t freed;                                                 /* Number of bytes successfully freed from memory                                           */
            do
            {
                freed = pbuf_free(ptr);                                 /* Free memory used by the packet that was just processed                                   */
            } while (freed == 0);

            tcp_recved(tpcb, plen);                                     /* Signal the remote host that more data can be received                                    */
        }
    }
}

/* Close function: closes a TCP connection and deallocates session resources */
void TcpClose(tcpPcb *tpcb, EchoSession *es)
{
    TCPServer_Com.step = 2;
    Ifx_Console_print("TCP is 'TcpClose()' \r\n");
    tcp_arg(tpcb, NULL);                                                /* Set the argument passed to every callback to NULL                                        */
    tcp_sent(tpcb, NULL);                                               /* Set the data sent callback to NULL                                                       */
    tcp_recv(tpcb, NULL);                                               /* Set the data received callback to NULL                                                   */
    tcp_err(tpcb, NULL);                                                /* Set the data error callback to NULL                                                      */
    tcp_poll(tpcb, NULL, 0);                                            /* Set the periodical callback to NULL                                                      */

    if (es != NULL)                                                     /* If a session still exists...                                                             */
    {
        mem_free(es);                                                   /* ...free memory assigned to the session.                                                  */
    }
    tcp_close(tpcb);                                                    /* Close the TCP connection                                                                 */
}

