

#ifndef TCP_SERVER_H_
#define TCP_SERVER_H_


#include <stdint.h>
#include <stdbool.h>
#include <math.h>
#include <string.h>

#include "Ifx_Console.h"
#include "lwip/opt.h"
#include "lwip/debug.h"
#include "lwip/stats.h"
#include "lwip/tcp.h"

#include "Ifx_Lwip.h"


/*********************************************************************************************************************/
/*------------------------------------------------------Macros-------------------------------------------------------*/
/*********************************************************************************************************************/
#define STORAGE_SIZE_BYTES           3000          /* Size in bytes of the space in memory allocated for storing incoming data */
#define TCP_CONNECT_PORT             8080
#define TCP_RECV_BUF_LENGTH_MAX      3000
#define RING_BUFFER_LENGTH_MAX       4000

/*********************************************************************************************************************/
/*-------------------------------------------------Data Structures---------------------------------------------------*/
/*********************************************************************************************************************/
enum EchoStates                         /* States of the session with the remote user                               */
{
    ES_NONE = 0,                        /* Session not completely initialized                                       */
    ES_ACCEPTED,                        /* Session assigned to a remote client and resources are being allocated    */
    ES_RECEIVING,                       /* Session is receiving data sent from remote client                        */
    ES_CLOSING                          /* Session is being closed and assigned resources will be deallocated       */
};

typedef struct tcp_pcb tcpPcb;          /* Define a more convenient type                                            */
typedef struct pbuf pBuf;               /* Define a more convenient type                                            */

typedef struct                          /* Session data structure used for communicating with a single remote client*/
{
    u8_t state;                         /* The current state for the session                                        */
    tcpPcb *pcb;                        /* Pointer to the TCP protocol control block used for this session          */
    pBuf *p;                            /* Pointer to the packet buffers used to store received packets             */
    char storage[STORAGE_SIZE_BYTES];   /* Storage for the received strings                                         */
    uint16 nextFreeStoragePos;          /* Position of the next free position in the storage array                  */
} EchoSession;

//typedef struct TcpServer_typ
//{
//    uint8_t step;  //1-tcp网络创建;2-client连接成功;3-接收到数据;
//    uint16_t port;
//    uint8_t RingBuffer[RING_BUFFER_LENGTH_MAX];
//    uint8_t ReceiveDataBuffer[TCP_RECIVE_BUFFER_LENGTH];
//    uint8_t SendDataBuffer[TCP_SEND_BUFFER_LENGTH];
//
//} TcpServer_typ;

typedef struct TCPServer_Com_typ
{
    tcpPcb *tcpPcb;
    tcpPcb *tcpPcbConnect;
    unsigned char step;  //1-tcp网络创建;2-client连接成功;3-接收到数据;4-断开连接
    unsigned short port;
    unsigned char OpenDelay;
    bool IdentPortChanged;
    unsigned long IdentWaitClosed;
    unsigned long IdentLastPort;
    bool TCP_Connected;
    bool TCP_SendDataPulse;
    unsigned char TCP_DataEndSignState;
    unsigned short TCP_RecvDataLength;
    unsigned short TCP_RecvDataCount;
    unsigned short TCP_RecvDataCount_Pre;
    unsigned short TCP_SendDataLength;
    unsigned short TCP_SendDataCount;
    unsigned short TCP_ClientVerifyResult;
    unsigned char TCP_SendBuffer[4000];
    unsigned char TCP_RecvBuffer[4000];
    unsigned short RingBuffer_HeadIndex;
    unsigned short RingBuffer_TailIndex;
    unsigned short RingBuffer_UntreatedLength;
    unsigned short RingBuffer_NoStartFlagCount;
    unsigned short RingBuffer_AnalyseResult;
    unsigned short FrameDataUpdateCount;
    unsigned long FrameEndSign;
    unsigned long FrameStartSign;
    unsigned char RingBuffer[4000];
} TCPServer_Com_typ;

extern struct TCPServer_Com_typ TCPServer_Com;

/*********************************************************************************************************************/
/*------------------------------------------------Function Prototypes------------------------------------------------*/
/*********************************************************************************************************************/
void TCP_Init(void);

void TCPServerConnect(void);


//void TcpRead(tcpPcb *newPcb, char *data, uint16_t length);

err_t TcpServerSend(tcpPcb *newPcb, unsigned char *data, uint16_t length);



#endif /* TCP_SERVER_H_ */
