
#include "UDP_Com.h"

struct UDPCom_typ UDPCom_GNSS;
unsigned long TickCount_1ms_Pre;


void UDP_Init(void)
{
    TickCount_1ms_Pre = g_TickCount_1ms;
    memset(&UDPCom_GNSS,0,sizeof(UDPCom_GNSS));

    //客户端地址设置
    IP4_ADDR(&UDPCom_GNSS.IPAddress_Client,192,168,20,159);
    IP4_ADDR(&UDPCom_GNSS.IPAddress_Server,192,168,20,111);
    UDPCom_GNSS.PortNum_Client = UDP_PORT_NUMBER_CLIENT;
    UDPCom_GNSS.PortNum_Server = UDP_PORT_NUMBER_SERVER;

    UDPCom_GNSS.UDPpcb = udp_new();
}

unsigned short UDP_Open(struct UDPCom_typ *com)
{
    unsigned short iReturn = 0;
    if (com->UDPpcb) //创建成功
    {
//        Ifx_Console_print("UDP New is successful! \r\n");
        err_t err = udp_connect(com->UDPpcb, &com->IPAddress_Client, com->PortNum_Client); //UDP客户端连接到指定IP地址和端口号的服务器
        if (err == ERR_OK)
        {
//            Ifx_Console_print("UDP connect is successful! \r\n");
            err = udp_bind(com->UDPpcb, IP_ADDR_ANY, com->PortNum_Server); //绑定本地IP地址与端口号

            if (err == ERR_OK)  //绑定完成
            {
//                Ifx_Console_print("UDP bind is successful! \r\n");
                udp_recv(com->UDPpcb, UDP_Recv, NULL); //注册接收回调函数
                iReturn = 1;
            }
            else
            {
                iReturn = 2;
            }
        }
        else
        {
            iReturn = 3;
        }
    }
    else
    {
        iReturn = 4;
    }
    return iReturn;
}

void UDP_Send(struct UDPCom_typ *com)
{
    struct pbuf *ptr;
    ptr = pbuf_alloc(PBUF_TRANSPORT, com->SendDataLen, PBUF_POOL); //申请内存

    if (ptr)
    {
        pbuf_take(ptr, (char *)com->SendDataBuffer, com->SendDataLen); //将tcp_demo_sendbuf中的数据打包进pbuf结构中
        udp_send(com->UDPpcb, ptr);    //udp发送数据
        pbuf_free(ptr);//释放内存
    }
}

void UDP_Recv(void *arg, struct udp_pcb *upcb, struct pbuf *p, const ip_addr_t *addr, u16_t port)
{
    unsigned short data_len = 0;
    struct pbuf *q;

    if (p != NULL)  //接收到不为空的数据时
    {
//        Ifx_Console_print("UDP receive data! \r\n");
        memset(&UDPCom_GNSS.RecvDataBuffer, 0, sizeof(UDPCom_GNSS.RecvDataBuffer)); //数据接收缓冲区清零

        for (q = p; q != NULL; q = q->next) //遍历完整个pbuf链表
        {
            //判断要拷贝到UDP_DEMO_RX_BUFSIZE中的数据是否大于UDP_DEMO_RX_BUFSIZE的剩余空间，如果大于
            //的话就只拷贝UDP_DEMO_RX_BUFSIZE中剩余长度的数据，否则的话就拷贝所有的数据
            if (q->len > (UDP_RECV_BUF_LENGTH_MAX - data_len))
            {
                memcpy(UDPCom_GNSS.RecvDataBuffer + data_len, q->payload, (UDP_RECV_BUF_LENGTH_MAX - data_len)); //拷贝数据
            }
            else
            {
                memcpy(UDPCom_GNSS.RecvDataBuffer + data_len, q->payload, q->len);

            }
            data_len += q->len;

            if (data_len > UDP_RECV_BUF_LENGTH_MAX)
            {
                data_len = UDP_RECV_BUF_LENGTH_MAX;
                break; //超出TCP客户端接收数组,跳出
            }
        }
        if(data_len > 0)
        {
            UDPCom_GNSS.RecvDataLen = data_len;
            UDPCom_GNSS.RecvDataCount ++;
        }

        //测试用，20221108
        UDPCom_GNSS.SendDataLen = UDPCom_GNSS.RecvDataLen;
        memcpy(UDPCom_GNSS.SendDataBuffer,UDPCom_GNSS.RecvDataBuffer,UDPCom_GNSS.SendDataLen);
        UDP_Send(&UDPCom_GNSS);

        Ifx_Console_print("UDP data length is %i,g_TickCount(%i),delta(%i ms),\r\n",UDPCom_GNSS.RecvDataLen,g_TickCount_1ms, g_TickCount_1ms-TickCount_1ms_Pre);
        TickCount_1ms_Pre = g_TickCount_1ms;
        upcb->remote_ip = *addr;                //记录远程主机的IP地址
        upcb->remote_port = port;           //记录远程主机的端口号

        pbuf_free(p);//释放内存
    }
    else
    {
        udp_disconnect(upcb);
    }
}



