#include "TCA9539.h"
#include "Bsp.h"
#include "Ifx_Types.h"
#include "IfxCpu.h"
#include "IfxScuWdt.h"

#define TCA9539_SCL_PIN IfxI2c1_SCL_P11_14_INOUT /* SCL PIN                                          */
#define TCA9539_SDA_PIN IfxI2c1_SDA_P11_13_INOUT /* SDA PIN                                          */
#define I2C_BAUDRATE 400000                      /* 400 kHz baud rate                                */

#define TCA9539_ADDR 0x77

#define INPUT_PORT_0 0x00
#define INPUT_PORT_1 0x01
#define OUTPUT_PORT_0 0x02
#define OUTPUT_PORT_1 0x03
#define POLARITY_INVERSION_PORT_0 0x04
#define POLARITY_INVERSION_PORT_1 0x05
#define CONFIGURATION_PORT_0 0x06
#define CONFIGURATION_PORT_1 0x07

#define TCA9539_1_RESET_PIN &MODULE_P23, 6
#define TCA9539_2_RESET_PIN &MODULE_P33, 3

IfxI2c_I2c g_i2cHandle;             /* I2C handle                                       */
IfxI2c_I2c_Device g_i2cDev_TCA9539; /* I2C Slave device handle to EEPROM of MC79411     */


void hard_delay(int times)
{
    int delay_num_ms = 0;
    for (int i = 0; i < times; i++)
    {
        delay_num_ms = 150000;
        while (delay_num_ms--)
          ;
    }
}

void init_I2C_module(void)
{
    /* Initialize module */
    IfxI2c_I2c_Config i2cConfig;                     /* Create configuration structure               */
    IfxI2c_I2c_initConfig(&i2cConfig, &MODULE_I2C1); /* Fill structure with default values and Module
                                                        address                                      */
    /* I2c pin configuration */
    const IfxI2c_Pins TCA9539_PINS =
        {
            &TCA9539_SCL_PIN,           /* SCL port pin                                 */
            &TCA9539_SDA_PIN,           /* SDA port pin                                 */
            IfxPort_PadDriver_ttlSpeed1 /* Pad driver mode                              */
        };
    i2cConfig.pins = &TCA9539_PINS;    /* Configure port pins                          */
    i2cConfig.baudrate = I2C_BAUDRATE; /* Configure baud rate with 400kHz              */

    IfxI2c_I2c_initModule(&g_i2cHandle, &i2cConfig); /* Initialize module                            */

    /* Initialize device */
    IfxI2c_I2c_deviceConfig i2cDeviceConfig;                     /* Create device configuration                  */
    IfxI2c_I2c_initDeviceConfig(&i2cDeviceConfig, &g_i2cHandle); /* Fill structure with default values and I2C
                                                                    Handler                                      */
    /* Because it is 7 bit long and bit 0 is R/W bit, the device address has to be shifted by 1 */
    i2cDeviceConfig.deviceAddress = TCA9539_ADDR << 1;
    IfxI2c_I2c_initDevice(&g_i2cDev_TCA9539, &i2cDeviceConfig); /* Initialize the I2C device handle             */
}

void TCA9539_init(void)
{
    uint8 buf[2];
//    uint8 ret = 0;
    init_I2C_module();

    //添加TCA9539复位引脚
    IfxPort_setPinModeOutput(TCA9539_1_RESET_PIN, IfxPort_OutputMode_pushPull, IfxPort_OutputIdx_general);
    IfxPort_setPinModeOutput(TCA9539_2_RESET_PIN, IfxPort_OutputMode_pushPull, IfxPort_OutputIdx_general);


    IfxPort_setPinHigh(TCA9539_1_RESET_PIN);
    IfxPort_setPinHigh(TCA9539_2_RESET_PIN);

    /* TCA9539 config P0 output*/
    buf[0] = CONFIGURATION_PORT_0;
    buf[1] = 0x00;
    while (IfxI2c_I2c_write(&g_i2cDev_TCA9539, buf, 2) == IfxI2c_I2c_Status_nak)
        ;

    /* TCA9539 config P1 output*/
    buf[0] = CONFIGURATION_PORT_1;
    buf[1] = 0x00;
    while (IfxI2c_I2c_write(&g_i2cDev_TCA9539, buf, 2) == IfxI2c_I2c_Status_nak)
        ;

    /* TCA9539 config P0 all Low*/
    buf[0] = OUTPUT_PORT_0;
    buf[1] = 0x00;
    while (IfxI2c_I2c_write(&g_i2cDev_TCA9539, buf, 2) == IfxI2c_I2c_Status_nak)
        ;

    /* TCA9539 config P1 all Low*/
    buf[0] = OUTPUT_PORT_1;
    buf[1] = 0x00;
    while (IfxI2c_I2c_write(&g_i2cDev_TCA9539, buf, 2) == IfxI2c_I2c_Status_nak)
        ;
    hard_delay(1000);
//    waitTime(500);

    /* TCA9539 config P0 all High*/
    buf[0] = OUTPUT_PORT_0;
    buf[1] = 0x8C;
    while (IfxI2c_I2c_write(&g_i2cDev_TCA9539, buf, 2) == IfxI2c_I2c_Status_nak)
        ;

    /* TCA9539 config P1 all High*/
    buf[0] = OUTPUT_PORT_1;
    buf[1] = 0xFF;
    while (IfxI2c_I2c_write(&g_i2cDev_TCA9539, buf, 2) == IfxI2c_I2c_Status_nak)
        ;
}
