/*********************************************************************************************************************/
/*-----------------------------------------------------Includes------------------------------------------------------*/
/*********************************************************************************************************************/
#include "tztek_adc.h"
#include "IfxEvadc_Adc.h"
#include "Bsp.h"
#include "Ifx_Console.h"
#include "UART_Logging.h"
#include <stdlib.h>

/*********************************************************************************************************************/
/*------------------------------------------------------Macros-------------------------------------------------------*/
/*********************************************************************************************************************/
#define GROUPID_9       IfxEvadc_GroupId_9                      /* EVADC group                                      */

#define AN1_CHID        4                                       /* Channel ID for pin AN1                           */
#define AN2_CHID        2                                       /* Channel ID for pin AN1                           */

/*********************************************************************************************************************/
/*-------------------------------------------------Global variables--------------------------------------------------*/
/*********************************************************************************************************************/
 /* EVADC handle */
IfxEvadc_Adc         g_evadc1;                                   /* EVADC module handle variable                     */
IfxEvadc_Adc_Group   g_adc1Group;                                /* EVADC group handle variable                      */
IfxEvadc_Adc_Channel g_adc1Channel;                              /* EVADC channel handle variable                    */

IfxEvadc_Adc         g_evadc2;                                   /* EVADC module handle variable                     */
IfxEvadc_Adc_Group   g_adc2Group;                                /* EVADC group handle variable                      */
IfxEvadc_Adc_Channel g_adc2Channel;                              /* EVADC channel handle variable                    */

Ifx_EVADC_G_RES g_result_1;                                       /* Variable to store the result of the measurement  */
Ifx_EVADC_G_RES g_result_2;                                       /* Variable to store the result of the measurement  */

/*********************************************************************************************************************/
/*------------------------------------------------Function Prototypes------------------------------------------------*/
/*********************************************************************************************************************/

/*********************************************************************************************************************/
/*---------------------------------------------Function Implementations----------------------------------------------*/
/*********************************************************************************************************************/


/* Function to initialize the EVADC with default parameters */
void tztek_adc_init(void)
{
    init_EVADC_1_Module();      /* Initialize the EVADC module  */
    init_EVADC_1_Group();       /* Initialize the EVADC group   */
    init_EVADC_1_Channels();    /* Initialize the channel       */
    fillAndStartQueue_1();    /* Fill the queue and start it  */

    // init_EVADC_2_Module();      /* Initialize the EVADC module  */
    // init_EVADC_2_Group();       /* Initialize the EVADC group   */
    init_EVADC_2_Channels();    /* Initialize the channel       */
    fillAndStartQueue_2();    /* Fill the queue and start it  */
}

/* Function to initialize the EVADC module with default parameters */
void init_EVADC_1_Module(void)
{
    /* Create configuration */
    IfxEvadc_Adc_Config adcConfig;
    IfxEvadc_Adc_initModuleConfig(&adcConfig, &MODULE_EVADC);

    /* Initialize module */
    IfxEvadc_Adc_initModule(&g_evadc1, &adcConfig);
}

/* Function to initialize the EVADC group with default parameters */
void init_EVADC_1_Group(void)
{
    /* Create and initialize group configuration with default values */
    IfxEvadc_Adc_GroupConfig adcGroupConfig;
    IfxEvadc_Adc_initGroupConfig(&adcGroupConfig, &g_evadc1);

    /* Setting user configuration using group 0 */
    adcGroupConfig.groupId = GROUPID_9;
    adcGroupConfig.master = GROUPID_9;

    /* Enable queued source */
    adcGroupConfig.arbiter.requestSlotQueue0Enabled = TRUE;

    /* Enable all gates in "always" mode (no edge detection) */
    adcGroupConfig.queueRequest[0].triggerConfig.gatingMode = IfxEvadc_GatingMode_always;

    /* Initialize the group */
    IfxEvadc_Adc_initGroup(&g_adc1Group, &adcGroupConfig);
}

/* Function to initialize the EVADC channel */
void init_EVADC_1_Channels(void)
{
    /* Create channel configuration */
    IfxEvadc_Adc_ChannelConfig adcChannelConfig;

    /* Initialize the configuration with default values */
    IfxEvadc_Adc_initChannelConfig(&adcChannelConfig, &g_adc1Group);

    /* Select the channel ID and the respective result register */
    adcChannelConfig.channelId = (IfxEvadc_ChannelId)(AN1_CHID);
    adcChannelConfig.resultRegister = (IfxEvadc_ChannelResult)(AN1_CHID);

    /* Initialize the channel */
    IfxEvadc_Adc_initChannel(&g_adc1Channel, &adcChannelConfig);
}

/* Function to add the channel to a queue */
void fillAndStartQueue_1(void)
{
    /* Add channel to queue with refill option enabled */
    IfxEvadc_Adc_addToQueue(&g_adc1Channel, IfxEvadc_RequestSource_queue0, IFXEVADC_QUEUE_REFILL);

    /* Start the queue */
    IfxEvadc_Adc_startQueue(&g_adc1Group, IfxEvadc_RequestSource_queue0);
}

/* Function to initialize the EVADC module with default parameters */
void init_EVADC_2_Module(void)
{
    /* Create configuration */
    IfxEvadc_Adc_Config adcConfig;
    IfxEvadc_Adc_initModuleConfig(&adcConfig, &MODULE_EVADC);

    /* Initialize module */
    IfxEvadc_Adc_initModule(&g_evadc2, &adcConfig);
}

/* Function to initialize the EVADC group with default parameters */
void init_EVADC_2_Group(void)
{
    /* Create and initialize group configuration with default values */
    IfxEvadc_Adc_GroupConfig adcGroupConfig;
    IfxEvadc_Adc_initGroupConfig(&adcGroupConfig, &g_evadc2);

    /* Setting user configuration using group 0 */
    adcGroupConfig.groupId = GROUPID_9;
    adcGroupConfig.master = GROUPID_9;

    /* Enable queued source */
    adcGroupConfig.arbiter.requestSlotQueue0Enabled = TRUE;

    /* Enable all gates in "always" mode (no edge detection) */
    adcGroupConfig.queueRequest[0].triggerConfig.gatingMode = IfxEvadc_GatingMode_always;

    /* Initialize the group */
    IfxEvadc_Adc_initGroup(&g_adc2Group, &adcGroupConfig);
}

/* Function to initialize the EVADC channel */
void init_EVADC_2_Channels(void)
{
    /* Create channel configuration */
    IfxEvadc_Adc_ChannelConfig adcChannelConfig;

    /* Initialize the configuration with default values */
    IfxEvadc_Adc_initChannelConfig(&adcChannelConfig, &g_adc1Group);

    /* Select the channel ID and the respective result register */
    adcChannelConfig.channelId = (IfxEvadc_ChannelId)(AN2_CHID);
    adcChannelConfig.resultRegister = (IfxEvadc_ChannelResult)(AN2_CHID);

    /* Initialize the channel */
    IfxEvadc_Adc_initChannel(&g_adc2Channel, &adcChannelConfig);
}

/* Function to add the channel to a queue */
void fillAndStartQueue_2(void)
{
    /* Add channel to queue with refill option enabled */
    IfxEvadc_Adc_addToQueue(&g_adc2Channel, IfxEvadc_RequestSource_queue0, IFXEVADC_QUEUE_REFILL);

    /* Start the queue */
    IfxEvadc_Adc_startQueue(&g_adc1Group, IfxEvadc_RequestSource_queue0);
}


/* Function to read the EVADC used channel */
void read_EVADC_1(void)
{
    Ifx_EVADC_G_RES conversionResult1;

    /* Wait for valid result */
    do
    {
        conversionResult1 = IfxEvadc_Adc_getResult(&g_adc1Channel); /* Read the result of the channel */
    } while(!conversionResult1.B.VF);

    /* Store result */
    g_result_1 = conversionResult1;
}

/* Function to read the EVADC used channel */
void read_EVADC_2(void)
{
    Ifx_EVADC_G_RES conversionResult2;

    /* Wait for valid result */
    do
    {
        conversionResult2 = IfxEvadc_Adc_getResult(&g_adc2Channel); /* Read the result of the channel */
    } while(!conversionResult2.B.VF);

    /* Store result */
    g_result_2 = conversionResult2;
}
void tztek_adc_test(void)
{
    int adc_value_change;
    static int cnt = 0;
    static int pre_adc_value1 = 0;
    static int pre_adc_value2 = 0;
    if (cnt == 60000)
    {
            /* code */
            read_EVADC_1();
            read_EVADC_2();
            adc_value_change = abs(g_result_1.B.RESULT - pre_adc_value1);
            if((g_result_1.B.RESULT >=  1000) && (adc_value_change >=  500))
            {
                // Ifx_Console_print("MCU_A_ADC_1 Value_change = %d\r\n",adc_value_change);
//                Ifx_Console_print("MCU_A_ADC_1 Value = %d\r\n",g_result_1.B.RESULT);
                if(((g_result_1.B.RESULT) > 1650 && (g_result_1.B.RESULT) < 1700)
                        || ((g_result_1.B.RESULT) > 2450 && (g_result_1.B.RESULT) < 2500)) {
                    Ifx_Console_print("MCU_A_ADC_1 Value is correct!\n");
                }
                pre_adc_value1 = g_result_1.B.RESULT;
            }
            adc_value_change = abs(g_result_2.B.RESULT - pre_adc_value2);
            if((g_result_2.B.RESULT >=  1000) && (adc_value_change >=  500))
            {
                // Ifx_Console_print("MCU_A_ADC_2 Value_change = %d\r\n",adc_value_change);
//                Ifx_Console_print("MCU_A_ADC_2 Value = %d\r\n",g_result_2.B.RESULT);
                if(((g_result_2.B.RESULT) > 1650 && (g_result_2.B.RESULT) < 1700)
                        || ((g_result_2.B.RESULT) > 2450 && (g_result_2.B.RESULT) < 2500)) {
                    Ifx_Console_print("MCU_A_ADC_2 Value is correct!\n");
                }
                pre_adc_value2 = g_result_2.B.RESULT;
            }
        cnt = 0;
    }
    cnt++;
    
}

