#include "tztek_gps.h"

static IfxAsclin_Asc g_asc_lin8_gps;
static IfxAsclin_Asc_Config ascConfig;
static uint8 g_asclin_gps_TxBuffer[ASC_TX_BUFFER_SIZE + sizeof(Ifx_Fifo) + 8];   /* Declaration of the FIFO parameters */
static uint8 g_asclin_gps_RxBuffer[ASC_RX_BUFFER_SIZE + sizeof(Ifx_Fifo) + 8];   /* Declaration of the FIFO parameters */


IFX_INTERRUPT(asclin8GpsTxISR, 0, INTPRIO_ASCLIN8_GPS_TX);      /* Adding the Interrupt Service Routine */
IFX_INTERRUPT(asclin8GpsRxISR, 0, INTPRIO_ASCLIN8_GPS_RX);      /* Adding the Interrupt Service Routine */
IFX_INTERRUPT(asclin8GpsErrISR, 0, INTPRIO_ASCLIN8_GPS_ER);     /* Adding the Interrupt Service Routine */

void asclin8GpsTxISR(void)
{
    IfxAsclin_Asc_isrTransmit(&g_asc_lin8_gps);
}

void asclin8GpsRxISR(void)
{
    IfxAsclin_Asc_isrReceive(&g_asc_lin8_gps);
}

void asclin8GpsErrISR(void)
{
    IfxAsclin_Asc_isrError(&g_asc_lin8_gps);
}

/***********************************************************************************************************************
 * Function Name : tztek_lin8_Init
 * Description   : UART8初始化
 * Arguments     :
 * Return Value  :
 * Author        : yinjun
 * Date          : 2022.9.27
 ***********************************************************************************************************************/
void tztek_lin8_Init()
{
    /* Initialize an instance of IfxAsclin_Asc_Config with default values */
    IfxAsclin_Asc_initModuleConfig(&ascConfig, &MODULE_ASCLIN1);


    /* Set the desired baud rate */
    ascConfig.baudrate.baudrate = SERIAL_BAUDRATE_9600;
    ascConfig.baudrate.oversampling = IfxAsclin_OversamplingFactor_16;

    /* ISR priorities and interrupt target */
    ascConfig.interrupt.txPriority = INTPRIO_ASCLIN8_GPS_TX;
    ascConfig.interrupt.rxPriority = INTPRIO_ASCLIN8_GPS_RX;
    ascConfig.interrupt.typeOfService = IfxCpu_Irq_getTos(IfxCpu_getCoreIndex());

    /* FIFO configuration */
    ascConfig.txBuffer = &g_asclin_gps_TxBuffer;
    ascConfig.txBufferSize = ASC_TX_BUFFER_SIZE;
    ascConfig.rxBuffer = &g_asclin_gps_RxBuffer;
    ascConfig.rxBufferSize = ASC_RX_BUFFER_SIZE;

    /* Pin configuration */
    const IfxAsclin_Asc_Pins pins =
    {
        NULL_PTR,       IfxPort_InputMode_pullUp,     /* CTS pin not used */
        &GPS_PIN_RX,   IfxPort_InputMode_pullUp,      /* RX pin           */
        NULL_PTR,       IfxPort_OutputMode_pushPull,  /* RTS pin not used */
        &GPS_PIN_TX,   IfxPort_OutputMode_pushPull,   /* TX pin           */
        IfxPort_PadDriver_cmosAutomotiveSpeed1
    };
    ascConfig.pins = &pins;

    IfxAsclin_Asc_initModule(&g_asc_lin8_gps, &ascConfig); /* Initialize module with above parameters */
}

/***********************************************************************************************************************
 * Function Name : tztek_SendMseeage_2_Gps
 * Description   : 通过串口发送数据
 * Arguments     :void *data：需要发送的数据；Ifx_SizeT count：发送数据的大小
 * Return Value  :
 * Author        : yinjun
 * Date          : 2022.9.27
 ***********************************************************************************************************************/
void tztek_SendMseeage_2_Gps(const void *data, Ifx_SizeT count)
{
    IfxAsclin_Asc_write(&g_asc_lin8_gps, data, &count, TIME_INFINITE);
}

/***********************************************************************************************************************
 * Function Name : mdelay
 * Description   : 延时函数，单位ms
 * Arguments     : uint32 delayTime:延时时间，单位ms
 * Return Value  :
 * Author        : yinjun
 * Date          : 2022.9.27
 ***********************************************************************************************************************/
static void mdelay(uint32 delayTime)
{
    waitTime(IfxStm_getTicksFromMilliseconds(BSP_DEFAULT_TIMER, delayTime));
}


/***********************************************************************************************************************
 * Function Name : tztek_gps_config
 * Description   : 配置GPS
 * Arguments     :
 * Return Value  :
 * Author        : yinjun
 * Date          : 2022.9.27
 ***********************************************************************************************************************/
void tztek_gps_config(void)
{
    mdelay(200);

    tztek_SendMseeage_2_Gps("\r\n", 2);
    mdelay(200);

    /* set gps baudrate */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_BAUD_115200, strlen(GPS_CMD_SET_BAUD_115200));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);

    /* Modify the lin8 baud rate */
    ascConfig.baudrate.baudrate = SERIAL_BAUDRATE_115200;
    IfxAsclin_Asc_initModule(&g_asc_lin8_gps, &ascConfig);
    mdelay(200);

    /* set frequency */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_FREQ, strlen(GPS_CMD_SET_FREQ));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);

    /* set msg */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_NMEA_MSG, strlen(GPS_CMD_SET_NMEA_MSG));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);

    /* set satellite */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_SATELLITE, strlen(GPS_CMD_SET_SATELLITE));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);

    /* set pro */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_PRO, strlen(GPS_CMD_SET_PRO));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);

    /* set reboot */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_HOT_REBOOT, strlen(GPS_CMD_SET_HOT_REBOOT));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);

    /* set mode */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_RUN_MODE, strlen(GPS_CMD_SET_RUN_MODE));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);

    /* set time */
    tztek_SendMseeage_2_Gps(GPS_CMD_SET_TIME, strlen(GPS_CMD_SET_TIME));
    mdelay(200);
    tztek_SendMseeage_2_Gps(GPS_CMD_SAVE, strlen(GPS_CMD_SAVE));
    mdelay(200);
}

