/**********************************************************************************************************************
 * \file ASCLIN_LIN_Master.c
 * \copyright Copyright (C) Infineon Technologies AG 2019
 *
 * Use of this file is subject to the terms of use agreed between (i) you or the company in which ordinary course of
 * business you are acting and (ii) Infineon Technologies AG or its licensees. If and as long as no such terms of use
 * are agreed, use of this file is subject to following:
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the software and
 * accompanying documentation covered by this license (the "Software") to use, reproduce, display, distribute, execute,
 * and transmit the Software, and to prepare derivative works of the Software, and to permit third-parties to whom the
 * Software is furnished to do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including the above license grant, this restriction
 * and the following disclaimer, must be included in all copies of the Software, in whole or in part, and all
 * derivative works of the Software, unless such copies or derivative works are solely in the form of
 * machine-executable object code generated by a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *********************************************************************************************************************/

/*********************************************************************************************************************/
/*-----------------------------------------------------Includes------------------------------------------------------*/
/*********************************************************************************************************************/
#include "tztek_lin.h"

#include "IfxAsclin.h"
#include "IfxAsclin_Lin.h"
#include "Ifx_Console.h"

/*********************************************************************************************************************/
/*------------------------------------------------------Macros-------------------------------------------------------*/
/*********************************************************************************************************************/
#define SERIAL_BAUDRATE     19200                               /* Set the baud rate                                */
#define SIZE                10                                  /* Size of the message                              */
#define ID_BYTE             0x80                                /* Set the LIN ID byte to a random number           */

#define LIN5_RX_PIN              IfxAsclin5_RXA_P00_6_IN             /* Set RX port pin                                  */
#define LIN5_TX_PIN              IfxAsclin5_TX_P00_7_OUT             /* Set TX port pin                                  */


#define LIN9_RX_PIN              IfxAsclin9_RXA_P01_5_IN             /* Set RX port pin                                  */
#define LIN9_TX_PIN              IfxAsclin9_TX_P01_7_OUT             /* Set TX port pin                                  */



/*********************************************************************************************************************/
/*-------------------------------------------------Global variables--------------------------------------------------*/
/*********************************************************************************************************************/
static IfxAsclin_Lin g_linMaster;
static IfxAsclin_Lin g_linSlaver;

unsigned char lin_rece_cont = 5;
unsigned char lin_trans_cont = 5;

/*********************************************************************************************************************/
/*----------------------------------------------Function Implementations---------------------------------------------*/
/*********************************************************************************************************************/
/* This function initializes the ASCLIN LIN module in master mode */
void init_ASCLIN_LIN_master(void)
{
    /* Initialize one instance of IfxAsclin_Lin_Config with default values */
    IfxAsclin_Lin_Config linMasterConfig;
    IfxAsclin_Lin_initModuleConfig(&linMasterConfig, &MODULE_ASCLIN5);

    linMasterConfig.linMode = IfxAsclin_LinMode_master;         /* Set the LIN mode of operation                    */
    linMasterConfig.brg.baudrate = SERIAL_BAUDRATE;             /* Set the desired baud rate                        */

    const IfxAsclin_Lin_Pins pins =
    {
        &LIN5_RX_PIN,   IfxPort_InputMode_pullUp,                    /* RX port pin                                      */
        &LIN5_TX_PIN,   IfxPort_OutputMode_pushPull,                 /* TX port pin                                      */
        IfxPort_PadDriver_cmosAutomotiveSpeed1
    };
    linMasterConfig.pins = &pins;                               /* Port pins configuration                          */

    /* Initialize module */
    IfxAsclin_Lin_initModule(&g_linMaster, &linMasterConfig);
}

void init_ASCLIN_LIN_slaver(void)
{
    /* Initialize one instance of IfxAsclin_Lin_Config with default values */
    IfxAsclin_Lin_Config linSlaverConfig;
    IfxAsclin_Lin_initModuleConfig(&linSlaverConfig, &MODULE_ASCLIN9);

    linSlaverConfig.linMode = IfxAsclin_LinMode_slave;         /* Set the LIN mode of operation                    */
    linSlaverConfig.brg.baudrate = SERIAL_BAUDRATE;             /* Set the desired baud rate                        */

    const IfxAsclin_Lin_Pins pins =
    {
        &LIN9_RX_PIN,   IfxPort_InputMode_pullUp,                    /* RX port pin                                      */
        &LIN9_TX_PIN,   IfxPort_OutputMode_pushPull,                 /* TX port pin                                      */
        IfxPort_PadDriver_cmosAutomotiveSpeed1
    };
    linSlaverConfig.pins = &pins;                               /* Port pins configuration                          */

    /* Initialize module */
    IfxAsclin_Lin_initModule(&g_linSlaver, &linSlaverConfig);
}

/* This function sends the string "Hello World!" */
void master_send_ASCLIN_LIN_message(void)
{
    uint8 txId = ID_BYTE;                                       /* ID which gets transmitted within the header      */
    uint8 txData[SIZE] = {"Lin9 ok"};                      /* Frame to send                                    */

    /* After the transmission of the LIN header by the master, the master itself starts the transmission
     * of the message.
     */

    IfxAsclin_Lin_sendHeader(&g_linMaster, &txId);              /* Send LIN header                                  */

    if(g_linMaster.acknowledgmentFlags.txHeaderEnd == 1)        /* If the LIN header has been transmitted           */
    {

        IfxAsclin_Lin_sendResponse(&g_linMaster, txData, SIZE); /* Send message                                     */

    }

    if(lin_trans_cont >0)
   {
       Ifx_Console_print("lin-5:trans ok!\n");
       lin_trans_cont--;
   }

}

void slaver_receive_ASCLIN_LIN_message(void)
{
    uint8 rxId = ID_BYTE;                                       /* ID which gets transmitted within the header      */
    uint8 rxData[SIZE] = {0};                      /* Frame to send                                    */

    /* After the transmission of the LIN header by the master, the master itself starts the transmission
     * of the message.
     */

    IfxAsclin_Lin_receiveHeader(&g_linSlaver, &rxId);              /* Send LIN header                                  */

    //Ifx_Console_print("receive header 0x%x\n",rxId);

    if(g_linSlaver.acknowledgmentFlags.rxHeaderEnd == 1)        /* If the LIN header has been transmitted           */
    {

        IfxAsclin_Lin_receiveResponse(&g_linSlaver, rxData, SIZE); /* Send message                                     */

    }
//    Ifx_Console_print("l-9: %s\n", rxData);

    if(lin_rece_cont > 0)
      {
          Ifx_Console_print("lin-9: %s\n", rxData);
          lin_rece_cont--;
      }

}


