#include "tztek_uart.h"
#include "IfxAsclin_Asc.h"
#include "Ifx_Fifo.h"
#include "IfxCpu_Irq.h"
#include "Ifx_Console.h"
#include "string.h"

/*********************************************************************************************************************/
/*------------------------------------------------------Macros-------------------------------------------------------*/
/*********************************************************************************************************************/
IfxAsclin_Asc g_asc_lin11;
IfxAsclin_Asc g_asc_lin6;
uint8 g_asclin_11TxBuffer[ASCLIN11_TX_BUFFER_SIZE + sizeof(Ifx_Fifo) + 8];   /* Declaration of the FIFO parameters */
uint8 g_asclin11_RxBuffer[ASCLIN11_RX_BUFFER_SIZE + sizeof(Ifx_Fifo) + 8];   /* Declaration of the FIFO parameters */
uint8 g_asclin_6TxBuffer[ASCLIN6_TX_BUFFER_SIZE + sizeof(Ifx_Fifo) + 8];     /* Declaration of the FIFO parameters */
uint8 g_asclin6_RxBuffer[ASCLIN6_RX_BUFFER_SIZE + sizeof(Ifx_Fifo) + 8];     /* Declaration of the FIFO parameters */

//IfxStdIf_DPipe  g_asclin11_StandardInterface;
//IfxStdIf_DPipe  g_asclin6_StandardInterface;

/*********************************************************************************************************************/
/*---------------------------------------------Function Implementations----------------------------------------------*/
/*********************************************************************************************************************/
IFX_INTERRUPT(asclin11TxISR, 0, INTPRIO_ASCLIN11_TX);    /* Adding the Interrupt Service Routine */
IFX_INTERRUPT(asclin6TxISR, 0, INTPRIO_ASCLIN6_TX);      /* Adding the Interrupt Service Routine */
IFX_INTERRUPT(asclin11RxISR, 0, INTPRIO_ASCLIN11_RX);
IFX_INTERRUPT(asclin6RxISR, 0, INTPRIO_ASCLIN6_RX);

void asclin11TxISR(void)
{
    IfxAsclin_Asc_isrTransmit(&g_asc_lin11);
}
void asclin11RxISR(void)
{
    IfxAsclin_Asc_isrReceive(&g_asc_lin11);
}
void asclin6TxISR(void)
{
    IfxAsclin_Asc_isrTransmit(&g_asc_lin6);
}
void asclin6RxISR(void)
{
    IfxAsclin_Asc_isrReceive(&g_asc_lin6);
}

/***********************************************************************************************************************
 * Function Name : tztek_uart_6_11_init
 * Description   : UART 6&11初始化
 * Arguments     :
 * Return Value  :
 * Author        : Joe
 * Date          : 2022.9.19
 ***********************************************************************************************************************/
void tztek_uart_6_11_init(void)
{
    /* Initialize an instance of IfxAsclin_Asc_Config with default values */
    IfxAsclin_Asc_Config ascConfig;
    IfxAsclin_Asc_initModuleConfig(&ascConfig, LIN6_PIN_RX.module);
    IfxAsclin_Asc_Config ascConfig11;
    IfxAsclin_Asc_initModuleConfig(&ascConfig11, LIN11_PIN_TX.module);
    /* Set the desired baud rate */
    ascConfig.baudrate.baudrate = SERIAL_LIN_BAUDRATE;
    ascConfig11.baudrate.baudrate = SERIAL_LIN_BAUDRATE;
//    ascConfig.baudrate.oversampling = IfxAsclin_OversamplingFactor_16; /* Set the oversampling factor */
    /* Configure the sampling mode */
//    ascConfig.bitTiming.medianFilter = IfxAsclin_SamplesPerBit_three;             /* Set the number of samples per bit*/
//    ascConfig.bitTiming.samplePointPosition = IfxAsclin_SamplePointPosition_8;    /* Set the first sample position    */
    /* ISR priorities and interrupt target */
    ascConfig.interrupt.txPriority = INTPRIO_ASCLIN6_TX;
    ascConfig.interrupt.rxPriority = INTPRIO_ASCLIN6_RX;
    ascConfig.interrupt.typeOfService = IfxCpu_Irq_getTos(IfxCpu_getCoreIndex());
    ascConfig11.interrupt.txPriority = INTPRIO_ASCLIN11_TX;
    ascConfig11.interrupt.rxPriority = INTPRIO_ASCLIN11_RX;
    ascConfig11.interrupt.typeOfService = IfxCpu_Irq_getTos(IfxCpu_getCoreIndex());
    /* FIFO configuration */
    ascConfig.txBuffer = &g_asclin_6TxBuffer;
    ascConfig.txBufferSize = ASCLIN6_TX_BUFFER_SIZE;
    ascConfig.rxBuffer = &g_asclin6_RxBuffer;
    ascConfig.rxBufferSize = ASCLIN6_RX_BUFFER_SIZE;
    ascConfig11.txBuffer = &g_asclin_11TxBuffer;
    ascConfig11.txBufferSize = ASCLIN11_TX_BUFFER_SIZE;
    ascConfig11.rxBuffer = &g_asclin11_RxBuffer;
    ascConfig11.rxBufferSize = ASCLIN11_RX_BUFFER_SIZE;
    /* Port pins configuration */
    const IfxAsclin_Asc_Pins pins =
    {
        NULL_PTR,         IfxPort_InputMode_pullUp,      /* CTS pin not used    */
        &LIN6_PIN_RX,   IfxPort_InputMode_pullUp,     /* RX pin              */
        NULL_PTR,         IfxPort_OutputMode_pushPull,   /* RTS pin not used    */
        &LIN6_PIN_TX,   IfxPort_OutputMode_pushPull,  /* TX pin              */
        IfxPort_PadDriver_cmosAutomotiveSpeed1
    };
    ascConfig.pins = &pins;
    /* Initialize module with above parameters  */
    IfxAsclin_Asc_initModule(&g_asc_lin6, &ascConfig);

    const IfxAsclin_Asc_Pins pins_11 =
    {
        NULL_PTR,         IfxPort_InputMode_pullUp,                     /* CTS pin not used                         */
        &LIN11_PIN_RX,   IfxPort_InputMode_pullUp,                     /* RX pin not used                          */
        NULL_PTR,         IfxPort_OutputMode_pushPull,                  /* RTS pin not used                         */
        &LIN11_PIN_TX,   IfxPort_OutputMode_pushPull,                  /* TX pin                                   */
        IfxPort_PadDriver_cmosAutomotiveSpeed1
    };
    ascConfig11.pins = &pins_11;

    IfxAsclin_Asc_initModule(&g_asc_lin11, &ascConfig11);  /* Initialize module with above parameters  */

    /* Initialize the Standard Interface */
//    IfxAsclin_Asc_stdIfDPipeInit(&g_asclin6_StandardInterface, &g_asc_lin6);
//    IfxAsclin_Asc_stdIfDPipeInit(&g_asclin11_StandardInterface, &g_asc_lin11);
    /* Initialize the Console */
//    Ifx_Console_init(&g_ascStandardInterface);
}

void tztek_uart_11_sendMessage(char * msg, Ifx_SizeT count)
{
    IfxAsclin_Asc_write(&g_asc_lin11, msg, &count, TIME_INFINITE);            /* Transfer of data                         */
}

/***********************************************************************************************************************
 * Function Name : tztek_uart_6_RingBack_Test
 * Description   : UART回环测试
 * Arguments     :
 * Return Value  :
 * Author        : Joe
 * Date          : 2022.9.19
 ***********************************************************************************************************************/
void tztek_uart_6_RingBack_Test(void)
{
    uint8   data[8];
    uint8 txData[] = {0x21, 0x55, 0x67, 0x71};
    static int cnt = 0;
    static int cnt2 = 0;
    uint8 cnt3 = 0;

    if(cnt == 60000)
    {
        if(cnt2 > 100) {
            waitTime(IfxStm_getTicksFromMilliseconds(BSP_DEFAULT_TIMER, 400));
            Ifx_SizeT readCount = (Ifx_SizeT)IfxAsclin_Asc_getReadCount(&g_asc_lin6);

            if(readCount > 0)
            {
                IfxAsclin_Asc_read(&g_asc_lin6, &data, &readCount, TIME_INFINITE);
                for(uint8 i = 0; i < readCount; i++)
                {
                    if(data[i] == txData[i])
                        cnt3++;
                    else
                        break;
                }
                if(cnt3 == readCount)
                    Ifx_Console_print("uart is ok!\n");
                else
                    Ifx_Console_print("uart is fail!\n");
                if(data[0] == 0x21)
                {
                    IfxAsclin_Asc_write(&g_asc_lin6, &data, &readCount, TIME_INFINITE);
                    waitTime(IfxStm_getTicksFromMilliseconds(BSP_DEFAULT_TIMER, 200));
                }
            }
            cnt2 = 0;
        }
        cnt2++;
        cnt = 0;
    }
    cnt++;
}
//void tztek_uart_6_RingBack_Test(void)
//{
//    uint8   data[8];
//    static int cnt = 0;
//    static int cnt2 = 0;
//
//
//    if(cnt == 60000)
//    {
//        if(cnt2 > 100) {
//            waitTime(IfxStm_getTicksFromMilliseconds(BSP_DEFAULT_TIMER, 400));
//            Ifx_SizeT readCount = (Ifx_SizeT)IfxAsclin_Asc_getReadCount(&g_asc_lin6);
//
//            if(readCount > 0)
//            {
//                IfxAsclin_Asc_read(&g_asc_lin6, &data, &readCount, TIME_INFINITE);
//                Ifx_Console_print("Asclin6_Slave read data : ");
//                for(uint8 i = 0; i < readCount; i++)
//                {
//                    Ifx_Console_print("%x ", data[i]);
//                }
//                Ifx_Console_print("\n");
//                if(data[0] == 0x21)
//                {
//                    IfxAsclin_Asc_write(&g_asc_lin6, &data, &readCount, TIME_INFINITE);
//                    waitTime(IfxStm_getTicksFromMilliseconds(BSP_DEFAULT_TIMER, 200));
//                }
//            }
//            cnt2 = 0;
//        }
//        cnt2++;
//        cnt = 0;
//    }
//    cnt++;
//}

/***********************************************************************************************************************
 * Function Name : tztek_uart_test
 * Description   : Log打印串口测试
 * Arguments     :
 * Return Value  :
 * Author        : Joe
 * Date          : 2022.9.22
 ***********************************************************************************************************************/
void tztek_uart_test(void)
{
    static int cnt = 0;
    char buf[100] = "RS232_2 send message ok\r\n";
    if (cnt == 60000)
    {
        tztek_uart_11_sendMessage(buf, 30);
        cnt = 0;
    }
    cnt++;

}

